/*****************************************************************************
 * FIRA_Throughput_21593_Core1.c
 *****************************************************************************/

/* Note : The numbers measured are with cache disabled */

#include <sys/platform.h>
#include "adi_initialize.h"
#include "FIRA_Throughput_21593_Core1.h"
#include <stdio.h>
#include <time.h>

volatile clock_t clock_start;
volatile clock_t clock_stop;
volatile clock_t cycles;

#pragma section("seg_l1_block0")
int FIRA_TCB[FIR_TCB_SIZE];

#pragma section("seg_l1_block1")
float InputBuff[MAX_TAP_LENGTH+MAX_WINDOW_SIZE-1];

#pragma section("seg_l1_block2")
float CoeffBuff[MAX_TAP_LENGTH];

#pragma section("seg_l1_block3")
float OutputBuff[MAX_WINDOW_SIZE];


void FIRA_Init(uint32_t uiFIR_GCTL, uint32_t* uiFIR_CHNPTR)
{
	*pREG_FIR0_CHNPTR=(uint32_t)uiFIR_CHNPTR;
	clock_start=clock();
	*pREG_FIR0_CTL1=uiFIR_GCTL;
}

uint32_t i = 0;
int main(int argc, char *argv[])
{

	uint32_t w; /* Window */;
	uint32_t t; /* Taps */
	/**
	 * Initialize managed drivers and/or services that have been added to 
	 * the project.
	 * @return zero on success 
	 */
	adi_initComponents();
	
	FILE * fpout;

	fpout = fopen("Results(Cycles).csv","w+");

	for(w=1;w<=MAX_WINDOW_SIZE;w++)
	{
		for(t=1;t<=MAX_TAP_LENGTH;t++)
		{
			//Initialize FIR TCBs for both the channels
			FIRA_TCB[0]=0;//((int)(FIRA_TCB2+12)>>2)|MP_OFFSET;	//CP
			FIRA_TCB[1]=t;	//CL
			FIRA_TCB[2]=1;	//CM
			FIRA_TCB[3]=((int)CoeffBuff>>2)|MP_OFFSET;	//CI
			FIRA_TCB[4]=((int)OutputBuff>>2)|MP_OFFSET;	//OB
			FIRA_TCB[5]=w;	//OL
			FIRA_TCB[6]=1;	//OM
			FIRA_TCB[7]=((int)OutputBuff>>2)|MP_OFFSET;	//OI
			FIRA_TCB[8]=((int)InputBuff>>2)|MP_OFFSET;	//IB
			FIRA_TCB[9]=t+w-1;	//IL
			FIRA_TCB[10]=1;	//IM
			FIRA_TCB[11]=((int)InputBuff>>2)|MP_OFFSET;	//II
			FIRA_TCB[12]=(t-1)|(w-1)<<BITP_FIR_CTL2_WINDOW;	//FIRCTL2

			//Disable accelerator
			*pREG_FIR0_CTL1 &= ~(BITM_FIR_CTL1_EN);

			//Start the IIR accelerator
			FIRA_Init(BITM_FIR_CTL1_EN|BITM_FIR_CTL1_DMAEN|(CHANNEL_NO-1)<<BITP_FIR_CTL1_CH|BITM_FIR_CTL1_BURSTEN|BITM_FIR_CTL1_BURST16_DIS|BITM_FIR_CTL1_PFB_EN,(uint32_t *)(((int)(FIRA_TCB+12)>>2)|MP_OFFSET));

			//wait till all channels are done
			while((*pREG_FIR0_DMASTAT & BITM_FIR_DMASTAT_ACDONE) == 0x00000000);

			clock_stop=clock();
			cycles=clock_stop-clock_start;

			//printf("%d\n",cycles);
			fprintf(fpout,"%d,",cycles);
	    }

		fprintf(fpout,"\n");
	}
	fclose(fpout);
    return 0;
}
